
(*
 * TrimDL - Trim DOWNLOAD.TXT file and produce report of
 *          unpopular and multiple-upload files.
 *
 * (C) 1987 Samuel H. Smith, 10-May-87 (rev. 16-Mar-89)
 *
 * This program is provided courtesy of:
 *         The Tool Shop
 *         Phoenix, Az
 *         (602) 279-2673
 *
 *
 * Disclaimer
 * ----------
 *
 * I cannot be responsible for any damages resulting from the use or mis-
 * use of this program!
 *
 * If you have any questions, bugs, or suggestions, please contact me at
 * The Tool Shop,  (602) 279-2673.
 *
 * Enjoy!     Samuel H. Smith
 *
 *)


{$M 32000,8000,500000}  {Stack, minheap, maxheap}
{$D+,L+}                {Produce debug information}
{$R-,S-}                {Rangecheck off, Stackcheck on}

uses dos, crt;

const
   version = 'v1.2 (03-17-89)';

type
   varstring = ^string;

   entry_ptr = ^entry_rec;
   entry_rec = record
      who:     varstring;
      date:    string[16];    {mm-dd-yyyy,hh:mm}
      fname:   string[12];
      ulcount: word;
      dlcount: word;
      higher:  entry_ptr;
      lower:   entry_ptr;
      exists:  boolean;
   end;

   actions = (output_new, output_unpop, output_dups);


const
   tot_uls:    word        = 0;
   tot_dls:    word        = 0;
   uniq_uls:   word        = 0;
   uniq_dls:   word        = 0;
   root:       entry_ptr   = nil;
   lnum:       word        = 0;
   found:      word        = 0;
   missing:    word        = 0;

var
   caction: actions;
   cur:     entry_rec;
   cline:   string;
   fd:      text;
   buf:     array[1..8704] of char;

type
   dirptr = ^dirinfo;
   dirinfo = record
      name: string[12];
      next: dirptr;
   end;

var
   dirfiles:   dirptr;


(* ------------------------------------------------------------ *)
procedure parse_line;
   {parse the current input line producing 'cur'}
var
   i:       integer;
   fun:     char;
   count:   word;
   str:     string;

begin
   readln(fd,cline);
   inc(lnum);
   str := cline;

   fun := str[2];

   str := copy(str,5,80);
   i := pos(',',str);
   cur.who^ := copy(str,1,i-1);

   str := copy(str,i+1,80);
   cur.date := copy(str,1,16);

   str := copy(str,18,80);
   i := pos(',',str);
   cur.fname := copy(str,1,i-1);

   str := copy(str,i+1,80);

   if str[1] = '?' then
      val(copy(str,5,20),count,i)
   else
      count := 1;

   cur.dlcount := 0;
   cur.ulcount := 0;
   if fun = 'D' then
      cur.dlcount := count
   else
      cur.ulcount := count;
end;


(* ------------------------------------------------------------ *)
procedure insert_node(var node: entry_ptr);
   {walk tree to find proper node for specified entry; expand tree as needed}
var
   dn:      integer;
   df:      dirptr;

begin
   if node = nil then   {new unique file}
   begin
      if memavail < 500 then
      begin
         writeln;
         writeln(^G'Need more memory to process your DOWNLOAD.TXT file!');
         writeln('At: ',cline);
         writeln;
         halt(1);
      end;

      new(node);
      getmem(node^.who,length(cur.who^)+1);
      node^.who^ := cur.who^;
      node^.date := cur.date;
      node^.fname := cur.fname;
      node^.ulcount := cur.ulcount;
      node^.dlcount := cur.dlcount;
      node^.higher := nil;
      node^.lower := nil;

      if cur.dlcount > 0 then
         inc(uniq_dls);
      if cur.ulcount > 0 then
         inc(uniq_uls);

      {scan directories for this file}
      df := dirfiles;
      while df <> nil do
      begin
         if df^.name = cur.fname then
         begin
            node^.exists := true;
            inc(found);
            exit;
         end;
         df := df^.next;
      end;

      {file not found in any of the dirs}
      node^.exists := false;
      inc(missing);
   end
   else

   {duplicate of existing entry}
   if cur.fname = node^.fname then
   begin
      node^.date := cur.date;
      inc(node^.ulcount,cur.ulcount);
      inc(node^.dlcount,cur.dlcount);
   end
   else

   {walk tree to find proper position}
   if cur.fname > node^.fname then
      insert_node(node^.higher)
   else
      insert_node(node^.lower);
end;


(* ------------------------------------------------------------ *)
procedure visit_node(node: entry_ptr);
begin
   if node^.exists then
   case caction of
   output_new:
      begin
         if node^.ulcount > 0 then
            writeln(fd,'(U),',
                    node^.who^,',',
                    node^.date,',',
                    node^.fname,',?,0,',
                    node^.ulcount);

         if node^.dlcount = 1 then
            writeln(fd,'(D),',
                    node^.who^,',',
                    node^.date,',',
                    node^.fname,',0,0,',
                    node^.dlcount);

         if node^.dlcount > 1 then
            writeln(fd,'(D),DOWNLOADED ',node^.dlcount,' TIMES,',
                    node^.date,',',
                    node^.fname,',?,0,',
                    node^.dlcount);
      end;

   output_unpop:
      if node^.dlcount = 0 then
         writeln(fd,
                 copy(node^.date,7,4),'/',
                 copy(node^.date,1,2),'/',   {yyyy/mm/dd,hh:mm format}
                 copy(node^.date,4,2),',',
                 copy(node^.date,12,5),' ',
                 node^.fname,'':12-length(node^.fname),' ',
                 node^.who^,'':25-length(node^.who^),
                 ' NO DOWNLOADS');

   output_dups:
      if node^.ulcount > 1 then
         writeln(fd,
                 copy(node^.date,7,4),'/',
                 copy(node^.date,1,2),'/',   {yyyy/mm/dd,hh:mm format}
                 copy(node^.date,4,2),',',
                 copy(node^.date,12,5),' ',
                 node^.fname,'':12-length(node^.fname),
                 '  U/L ',node^.ulcount,' TIMES');
   end;
end;


(* ------------------------------------------------------------ *)
procedure walk_tree(node: entry_ptr);
   {walk tree to and perform specified action at each node}
   {note: tree is NOT walked in sort order - otherwise it will not
    be regenerated as a true tree on the next run!}
begin
   if node = nil then
      exit;

   visit_node(node);
   walk_tree(node^.lower);
   walk_tree(node^.higher);
end;


(* ------------------------------------------------------------ *)
procedure produce_report(action: actions;
                         what:   string;
                         fname:  string);
begin
   writeln('Writing ',what,' to "',fname,'" ...');
   assign(fd,fname);
   rewrite(fd);
   setTextBuf(fd,buf);
   caction := action;
   walk_tree(root);
   close(fd);
end;


(* ------------------------------------------------------------ *)
procedure usage;
begin
   writeln('You must specify a configuration file on the command line.');
   writeln;
   writeln('Example:  trimdl trimdl.cnf');
   writeln;
   writeln('The config file must list ALL download directories, one per line.');
   halt;
end;


(* ------------------------------------------------------------ *)
procedure load_config;
var
   fd:      text;
   line:    string;
   DirInfo: SearchRec;
   df:      dirptr;
   num:     word;

begin
   assign(fd,paramstr(1));
   {$i-} reset(fd); {$i+}
   if ioresult <> 0 then
   begin
      writeln('Can''t read configuration file: ',paramstr(1));
      halt;
   end;

   num := 0;
   dirfiles := nil;
   while not eof(fd) do
   begin
      readln(fd,line);

      if line[length(line)] = '\' then
         dec(line[0]);

      write(^M'Scanning ',line); clreol;
      FindFirst(line+'\*.*',$21,DirInfo);
      while DosError = 0 do
      begin
         new(df);
         df^.next := dirfiles;
         df^.name := DirInfo.name;
         dirfiles := df;
         FindNext(DirInfo);
         inc(num);
      end;
   end;

   write(^M,num,' files in download directories.');
   clreol;
   writeln;
   close(fd);
end;


(* ------------------------------------------------------------ *)
begin
   writeln;
   writeln('TrimDL ',version,'  Copyright (C) 1987, 1989 Samuel H. Smith');
   writeln('Courtesy of:  The Tool Shop,  (602) 264-3969 / (602) 279-2673');
   writeln;

   writeln('This utility compresses your DOWNLOAD.TXT file by "trimming" out');
   writeln('multiple download entries for the same file.  It also generates a');
   writeln('report on unpopular files and files uploaded more than once.');
   writeln;

   if paramcount <> 1 then
      usage;

   load_config;

   getmem(cur.who,80);
   writeln('Loading DOWNLOAD.TXT ...');

   assign(fd,'DOWNLOAD.TXT');
   {$i-} reset(fd); {$i+}
   if ioresult <> 0 then
   begin
      writeln('Can''t open DOWNLOAD.TXT');
      halt;
   end;

   setTextBuf(fd,buf);
   while not eof(fd) do
   begin
      parse_line;
      insert_node(root);
      inc(tot_dls,cur.dlcount);
      inc(tot_uls,cur.ulcount);
      write(^M,tot_uls,' U/L (',uniq_uls,' unique),  ',
               tot_dls,' D/L (',uniq_dls,' unique),  ',
               missing,' missing,  ',
               found,' found');
   end;
   close(fd);

   writeln(^M,tot_uls,' U/L (',uniq_uls,' unique),  ',
              tot_dls,' D/L (',uniq_dls,' unique),  ',
              missing,' missing,  ',
              found,' found');

   writeln;
   produce_report(output_new,'trimmed DOWNLOAD listing','DOWNLOAD.NEW');
   produce_report(output_unpop,'list of files never downloaded','UNPOP');
   produce_report(output_dups,'list of files uploaded more than once','DUPS');
   writeln;
end.

